import React, { Suspense } from 'react';
import ReactDOM from 'react-dom/client';
import { createBrowserRouter, RouterProvider } from 'react-router-dom';
import { QueryClient, QueryClientProvider } from '@tanstack/react-query';
import { ThemeProvider } from '@/components/ui/theme-provider';
import { AuthProvider } from '@/contexts/AuthContext';
import { UnifiedSettingsProvider } from '@/contexts/UnifiedSettingsContext';
import AuthErrorBoundary from '@/components/AuthErrorBoundary';
import ErrorBoundary from '@/components/ErrorBoundary';
import { Toaster } from '@/components/ui/toaster';
import PWAInstallPrompt from '@/components/pwa/PWAInstallPrompt';
import OfflineIndicator from '@/components/pwa/OfflineIndicator';
import { usePWAInit } from '@/hooks/usePWAInit';
import { registerServiceWorker } from '@/utils/pwaUtils';
import './index.css';

// Lazy load all pages for code splitting
const Index = React.lazy(() => import('@/pages/Index'));
const Auth = React.lazy(() => import('@/pages/Auth'));
const Discover = React.lazy(() => import('@/pages/Discover'));
const Books = React.lazy(() => import('@/pages/Books'));
const BookDetails = React.lazy(() => import('@/pages/BookDetails'));
const EditionDetails = React.lazy(() => import('@/pages/EditionDetails'));
const Authors = React.lazy(() => import('@/pages/Authors'));
const AuthorPage = React.lazy(() => import('@/pages/AuthorPage'));
const Series = React.lazy(() => import('@/pages/Series'));
const SeriesPage = React.lazy(() => import('@/pages/SeriesPage'));
const Narrators = React.lazy(() => import('@/pages/Narrators'));
const NarratorPage = React.lazy(() => import('@/pages/NarratorPage'));
const Search = React.lazy(() => import('@/pages/Search'));
const Dashboard = React.lazy(() => import('@/pages/Dashboard'));
const MatchWithBook = React.lazy(() => import('@/pages/MatchWithBook'));
const MyShelves = React.lazy(() => import('@/pages/MyShelves'));
const MyLibraryBooks = React.lazy(() => import('@/pages/MyLibraryBooks'));
const MyARCs = React.lazy(() => import('@/pages/MyARCs'));
const AddARC = React.lazy(() => import('@/pages/AddARC'));
const BuddyReads = React.lazy(() => import('@/pages/BuddyReads'));
const BuddyReadDetails = React.lazy(() => import('@/pages/BuddyReadDetails'));
const ReadingJournal = React.lazy(() => import('@/pages/ReadingJournal'));
const ReadingStats = React.lazy(() => import('@/pages/ReadingStats'));
const UserProfile = React.lazy(() => import('@/pages/UserProfile'));
const ProfileEdit = React.lazy(() => import('@/pages/ProfileEdit'));
const SocialSettings = React.lazy(() => import('@/pages/SocialSettings'));
const NotificationCenter = React.lazy(() => import('@/pages/NotificationCenter'));
const LibrarianPanel = React.lazy(() => import('@/pages/LibrarianPanel'));
const AdminPanel = React.lazy(() => import('@/pages/AdminPanel'));
const AdminRoute = React.lazy(() => import('@/components/AdminRoute'));
const ModerationPanel = React.lazy(() => import('@/pages/ModerationPanel'));
const MyOwnedBooks = React.lazy(() => import('@/pages/MyOwnedBooks'));
const DebugOwnedBooks = React.lazy(() => import('@/components/ownedBooks/DebugOwnedBooks'));
const OwnedBooksDebugger = React.lazy(() => import('@/components/ownedBooks/OwnedBooksDebugger'));
const TestOwnedBooks = React.lazy(() => import('@/pages/TestOwnedBooks'));
const DatabaseTest = React.lazy(() => import('@/components/ownedBooks/DatabaseTest'));
const MigrationTest = React.lazy(() => import('@/components/ownedBooks/MigrationTest'));
const ErrorLogger = React.lazy(() => import('@/components/ownedBooks/ErrorLogger'));
const LibraryDashboard = React.lazy(() => import('@/pages/LibraryDashboard'));
const OPACCatalogPage = React.lazy(() => import('@/pages/OPACCatalogPage'));
const PublicCatalogPage = React.lazy(() => import('@/pages/PublicCatalogPage'));
const CirculationPage = React.lazy(() => import('@/pages/CirculationPage'));
const InventoryPage = React.lazy(() => import('@/pages/InventoryPage'));
const LibraryAutomationPage = React.lazy(() => import('@/pages/LibraryAutomationPage'));
const MyCalendar = React.lazy(() => import('@/pages/MyCalendar'));
const NotFound = React.lazy(() => import('@/pages/NotFound'));
const Feed = React.lazy(() => import('@/pages/Feed'));
const Lists = React.lazy(() => import('@/pages/Lists'));
const BookListsDashboard = React.lazy(() => import('@/pages/BookListsDashboard'));
const BookListDetails = React.lazy(() => import('@/pages/BookListDetails'));
const ReadingBuddiesManagerPage = React.lazy(() => import("@/pages/ReadingBuddiesManager"));
const ChallengesDirectory = React.lazy(() => import('@/pages/ChallengesDirectory'));
const ChallengeCreateEdit = React.lazy(() => import('@/pages/ChallengeCreateEdit'));
const ChallengeDetails = React.lazy(() => import('@/pages/ChallengeDetails'));
const MyPreOrders = React.lazy(() => import('@/pages/MyPreOrders'));
const MyUpNext = React.lazy(() => import('@/pages/MyUpNext'));
const AdvancedAnalytics = React.lazy(() => import('@/pages/AdvancedAnalytics'));
const DashboardCustomization = React.lazy(() => import('@/pages/DashboardCustomization'));
const BugTracker = React.lazy(() => import('@/pages/BugTracker'));
const BugReport = React.lazy(() => import('@/pages/BugReport'));
const BugDetails = React.lazy(() => import('@/pages/BugDetails'));
const HelpCenter = React.lazy(() => import('@/pages/HelpCenter'));
const SupportTickets = React.lazy(() => import('@/pages/SupportTickets'));
const NewSupportTicket = React.lazy(() => import('@/pages/NewSupportTicket'));
const SupportTicketDetails = React.lazy(() => import('@/pages/SupportTicketDetails'));
const HollowCup = React.lazy(() => import('@/pages/HollowCup'));
const PointsGuide = React.lazy(() => import('@/pages/PointsGuide'));
const HousePage = React.lazy(() => import('@/pages/HousePage'));
const NotificationDebug = React.lazy(() => import('@/pages/NotificationDebug'));
const ReadingGoals = React.lazy(() => import('@/pages/ReadingGoals'));
const FollowedAuthors = React.lazy(() => import('@/pages/FollowedAuthors'));
const TestFollowAuthors = React.lazy(() => import('@/pages/TestFollowAuthors'));
const SocialSharingTest = React.lazy(() => import('@/pages/SocialSharingTest'));
const EditTest = React.lazy(() => import('@/pages/EditTest'));
const AjaxTest = React.lazy(() => import('@/pages/AjaxTest'));
const Achievements = React.lazy(() => import('@/pages/Achievements'));
const SeriesImageDemo = React.lazy(() => import('@/pages/SeriesImageDemo'));
const Blog = React.lazy(() => import('@/pages/Blog'));
const BlogPost = React.lazy(() => import('@/pages/BlogPost'));
const Forums = React.lazy(() => import('@/pages/Forums'));
const ForumBoard = React.lazy(() => import('@/pages/ForumBoard'));
const ForumTopic = React.lazy(() => import('@/pages/ForumTopic'));
const CreateForumTopic = React.lazy(() => import('@/pages/CreateForumTopic'));
const ReadingDatesTest = React.lazy(() => import('@/pages/ReadingDatesTest'));

// Loading component for Suspense
const PageLoading = () => (
  <div className="min-h-screen bg-background flex items-center justify-center">
    <div className="text-center space-y-4">
      <div className="animate-spin rounded-full h-12 w-12 border-b-2 border-primary mx-auto"></div>
      <p className="text-muted-foreground">Loading...</p>
    </div>
  </div>
);

// Wrapper component for lazy-loaded pages
const LazyPage = ({ children }: { children: React.ReactNode }) => (
  <Suspense fallback={<PageLoading />}>
    {children}
  </Suspense>
);

const queryClient = new QueryClient({
  defaultOptions: {
    queries: {
      staleTime: 5 * 60 * 1000, // 5 minutes
      gcTime: 10 * 60 * 1000, // 10 minutes (formerly cacheTime)
      refetchOnWindowFocus: false,
      refetchOnReconnect: true,
      retry: (failureCount, error) => {
        // Don't retry on 4xx errors
        if (error && typeof error === 'object' && 'status' in error) {
          const status = (error as any).status;
          if (status >= 400 && status < 500) {
            return false;
          }
        }
        // Retry up to 3 times for other errors
        return failureCount < 3;
      },
      // Optimize for mobile networks
      networkMode: 'online',
    },
    mutations: {
      retry: 1,
      networkMode: 'online',
    },
  },
});

const router = createBrowserRouter([
  {
    path: '/',
    element: <LazyPage><Index /></LazyPage>,
  },
  {
    path: '/auth',
    element: <LazyPage><Auth /></LazyPage>,
  },
  {
    path: '/discover',
    element: <LazyPage><Discover /></LazyPage>,
  },
  {
    path: '/books',
    element: <LazyPage><Books /></LazyPage>,
  },
  {
    path: '/books/:id',
    element: <LazyPage><BookDetails /></LazyPage>,
  },
  {
    path: '/books/editions/:editionId',
    element: <LazyPage><EditionDetails /></LazyPage>,
  },
  {
    path: '/editions/:editionId',
    element: <LazyPage><EditionDetails /></LazyPage>,
  },
  {
    path: '/authors',
    element: <LazyPage><Authors /></LazyPage>,
  },
  {
    path: '/authors/:authorSlug',
    element: <LazyPage><AuthorPage /></LazyPage>,
  },
  {
    path: '/series',
    element: <LazyPage><Series /></LazyPage>,
  },
  {
    path: '/series/:seriesSlug',
    element: <LazyPage><SeriesPage /></LazyPage>,
  },
  {
    path: '/narrators',
    element: <LazyPage><Narrators /></LazyPage>,
  },
  {
    path: '/narrators/:narratorSlug',
    element: <LazyPage><NarratorPage /></LazyPage>,
  },
  {
    path: '/search',
    element: <LazyPage><Search /></LazyPage>,
  },
  {
    path: '/dashboard',
    element: <LazyPage><Dashboard /></LazyPage>,
  },
  {
    path: '/dashboard/customize',
    element: <LazyPage><DashboardCustomization /></LazyPage>
  },
  {
    path: '/match-with-book',
    element: <LazyPage><MatchWithBook /></LazyPage>,
  },
  {
    path: '/my-shelves',
    element: <LazyPage><MyShelves /></LazyPage>,
  },
  {
    path: '/my-library-books',
    element: <LazyPage><MyLibraryBooks /></LazyPage>,
  },
  {
    path: '/arcs',
    element: <LazyPage><MyARCs /></LazyPage>,
  },
  {
    path: '/arcs/add',
    element: <LazyPage><AddARC /></LazyPage>,
  },
  {
    path: '/my-owned-books',
    element: <LazyPage><MyOwnedBooks /></LazyPage>,
  },
  {
    path: '/debug-owned-books',
    element: <LazyPage><DebugOwnedBooks /></LazyPage>,
  },
  {
    path: '/debug-owned-books-advanced',
    element: <LazyPage><OwnedBooksDebugger /></LazyPage>,
  },
  {
    path: '/test-owned-books',
    element: <LazyPage><TestOwnedBooks /></LazyPage>,
  },
  {
    path: '/database-test',
    element: <LazyPage><DatabaseTest /></LazyPage>,
  },
  {
    path: '/migration-test',
    element: <LazyPage><MigrationTest /></LazyPage>,
  },
  {
    path: '/error-logger',
    element: <LazyPage><ErrorLogger /></LazyPage>,
  },
  {
    path: '/library',
    element: <LazyPage><LibraryDashboard /></LazyPage>,
  },
  {
    path: '/library/catalog',
    element: <LazyPage><OPACCatalogPage /></LazyPage>,
  },
  {
    path: '/library/circulation',
    element: <LazyPage><CirculationPage /></LazyPage>,
  },
  {
    path: '/library/inventory',
    element: <LazyPage><InventoryPage /></LazyPage>,
  },
  {
    path: '/library/automation',
    element: <LazyPage><LibraryAutomationPage /></LazyPage>,
  },
  {
    path: '/catalog/:userId',
    element: <LazyPage><PublicCatalogPage /></LazyPage>,
  },
  {
    path: '/my-calendar',
    element: <LazyPage><MyCalendar /></LazyPage>,
  },
  {
    path: '/my-pre-orders',
    element: <LazyPage><MyPreOrders /></LazyPage>,
  },
  {
    path: '/my-up-next',
    element: <LazyPage><MyUpNext /></LazyPage>,
  },
  {
    path: '/hollow-cup',
    element: <LazyPage><HollowCup /></LazyPage>,
  },
  {
    path: '/hollow-cup/points-guide',
    element: <LazyPage><PointsGuide /></LazyPage>,
  },
  {
    path: '/house/:houseId',
    element: <LazyPage><HousePage /></LazyPage>,
  },
  {
    path: '/buddy-reads',
    element: <LazyPage><BuddyReads /></LazyPage>,
  },
  {
    path: '/buddy-reads/:buddyReadId',
    element: <LazyPage><BuddyReadDetails /></LazyPage>,
  },
  {
    path: '/reading-journal',
    element: <LazyPage><ReadingJournal /></LazyPage>,
  },
  {
    path: '/reading-stats',
    element: <LazyPage><ReadingStats /></LazyPage>,
  },
  {
    path: '/reading-goals',
    element: <LazyPage><ReadingGoals /></LazyPage>,
  },
  {
    path: '/followed-authors',
    element: <LazyPage><FollowedAuthors /></LazyPage>,
  },
  {
    path: '/test-follow-authors',
    element: <LazyPage><TestFollowAuthors /></LazyPage>,
  },
  {
    path: '/test-social-sharing',
    element: <LazyPage><SocialSharingTest /></LazyPage>,
  },
  {
    path: '/edit-test',
    element: <LazyPage><EditTest /></LazyPage>,
  },
  {
    path: '/ajax-test',
    element: <LazyPage><AjaxTest /></LazyPage>,
  },
  {
    path: '/achievements',
    element: <LazyPage><Achievements /></LazyPage>,
  },
  {
    path: '/blog',
    element: <LazyPage><Blog /></LazyPage>,
  },
  {
    path: '/blog/:slug',
    element: <LazyPage><BlogPost /></LazyPage>,
  },
  {
    path: '/forums',
    element: <LazyPage><Forums /></LazyPage>,
  },
  {
    path: '/forums/board/:boardSlug',
    element: <LazyPage><ForumBoard /></LazyPage>,
  },
  {
    path: '/forums/:boardSlug',
    element: <LazyPage><ForumBoard /></LazyPage>,
  },
  {
    path: '/forums/board/:boardSlug/new-topic',
    element: <LazyPage><CreateForumTopic /></LazyPage>,
  },
  {
    path: '/forums/board/:boardSlug/:topicSlug',
    element: <LazyPage><ForumTopic /></LazyPage>,
  },
  {
    path: '/forums/:boardSlug/:topicSlug',
    element: <LazyPage><ForumTopic /></LazyPage>,
  },
  {
    path: '/series-image-demo',
    element: <LazyPage><SeriesImageDemo /></LazyPage>,
  },
  {
    path: '/profile/:username',
    element: <LazyPage><UserProfile /></LazyPage>,
  },
  {
    path: '/profile/:username/stats',
    element: <LazyPage><ReadingStats /></LazyPage>,
  },
  {
    path: '/profile/edit',
    element: <LazyPage><ProfileEdit /></LazyPage>,
  },
  {
    path: '/profile-edit',
    element: <LazyPage><ProfileEdit /></LazyPage>,
  },
  {
    path: '/social-settings',
    element: <LazyPage><SocialSettings /></LazyPage>,
  },
  {
    path: '/notifications',
    element: <LazyPage><NotificationCenter /></LazyPage>,
  },
  {
    path: '/librarian',
    element: <LazyPage><AdminRoute><LibrarianPanel /></AdminRoute></LazyPage>,
  },
  {
    path: '/admin',
    element: <LazyPage><AdminRoute><AdminPanel /></AdminRoute></LazyPage>,
  },
  {
    path: '/moderation',
    element: <LazyPage><AdminRoute><ModerationPanel /></AdminRoute></LazyPage>,
  },
  {
    path: '/feed',
    element: <LazyPage><Feed /></LazyPage>,
  },
  {
    path: '/lists',
    element: <LazyPage><Lists /></LazyPage>,
  },
  {
    path: '/lists/:listId',
    element: <LazyPage><BookListDetails /></LazyPage>,
  },
  {
    path: '/my-lists',
    element: <LazyPage><BookListsDashboard /></LazyPage>,
  },
  {
    path: '/reading-buddies',
    element: <LazyPage><ReadingBuddiesManagerPage /></LazyPage>,
  },
  {
    path: "/challenges",
    element: <LazyPage><ChallengesDirectory /></LazyPage>
  },
  {
    path: "/challenges/create",
    element: <LazyPage><ChallengeCreateEdit /></LazyPage>
  },
  {
    path: "/challenges/:challengeId",
    element: <LazyPage><ChallengeDetails /></LazyPage>
  },
  {
    path: "/analytics/advanced",
    element: <LazyPage><AdvancedAnalytics /></LazyPage>
  },
  {
    path: '/bug-tracker',
    element: <LazyPage><BugTracker /></LazyPage>,
  },
  {
    path: '/bug-tracker/report',
    element: <LazyPage><BugReport /></LazyPage>,
  },
  {
    path: '/bug-tracker/:id',
    element: <LazyPage><BugDetails /></LazyPage>,
  },
  {
    path: '/help',
    element: <LazyPage><HelpCenter /></LazyPage>,
  },
  {
    path: '/support',
    element: <LazyPage><SupportTickets /></LazyPage>,
  },
  {
    path: '/support/new',
    element: <LazyPage><NewSupportTicket /></LazyPage>,
  },
  {
    path: '/support/:id',
    element: <LazyPage><SupportTicketDetails /></LazyPage>,
  },
  {
    path: '/debug/notifications',
    element: <LazyPage><NotificationDebug /></LazyPage>,
  },
  {
    path: '/reading-dates-test',
    element: <LazyPage><ReadingDatesTest /></LazyPage>,
  },
  {
    path: '*',
    element: <LazyPage><NotFound /></LazyPage>,
  },
]);

// Global error handler for unhandled promise rejections
window.addEventListener('unhandledrejection', (event) => {
  console.error('Unhandled promise rejection:', event.reason);
  // Prevent the default browser behavior
  event.preventDefault();
});

// Global error handler for uncaught errors
window.addEventListener('error', (event) => {
  console.error('Uncaught error:', event.error);
  // Prevent the default browser behavior
  event.preventDefault();
});

const rootElement = document.getElementById('root');
if (!rootElement) {
  throw new Error('Root element not found');
}

// PWA Initialization Component
const PWAInitializer: React.FC = () => {
  const { isReady } = usePWAInit();

  React.useEffect(() => {
    // Register service worker
    registerServiceWorker().catch(console.error);
  }, []);

  if (!isReady) {
    return null;
  }

  return (
    <>
      <PWAInstallPrompt />
      <OfflineIndicator />
    </>
  );
};

ReactDOM.createRoot(rootElement).render(
  <React.StrictMode>
    <ErrorBoundary>
      <AuthErrorBoundary>
        <QueryClientProvider client={queryClient}>
          <ThemeProvider defaultTheme="dark" storageKey="reading-hollow-theme">
            <AuthProvider>
              <UnifiedSettingsProvider>
                <RouterProvider router={router} />
                <PWAInitializer />
              </UnifiedSettingsProvider>
            </AuthProvider>
            <Toaster />
          </ThemeProvider>
        </QueryClientProvider>
      </AuthErrorBoundary>
    </ErrorBoundary>
  </React.StrictMode>,
);
